@@ -1153,17 +1153,31 @@ static bool Update(wchar_t *cmdLine)
 		GetCurrentDirectory(_countof(lpAppDataPath), lpAppDataPath);
 		StringCbCat(lpAppDataPath, sizeof(lpAppDataPath), L"\\config");
 	} else {
-		CoTaskMemPtr<wchar_t> pOut;
-		HRESULT hr = SHGetKnownFolderPath(FOLDERID_RoamingAppData,
-						  KF_FLAG_DEFAULT, nullptr,
-						  &pOut);
-		if (hr != S_OK) {
+		DWORD ret;
+		ret = GetEnvironmentVariable(L"OBS_USER_APPDATA_PATH",
+					     lpAppDataPath,
+					     _countof(lpAppDataPath));
+
+		if (ret >= _countof(lpAppDataPath)) {
 			Status(L"Update failed: Could not determine AppData "
 			       L"location");
 			return false;
 		}
 
-		StringCbCopy(lpAppDataPath, sizeof(lpAppDataPath), pOut);
+		if (!ret) {
+			CoTaskMemPtr<wchar_t> pOut;
+			HRESULT hr = SHGetKnownFolderPath(
+				FOLDERID_RoamingAppData, KF_FLAG_DEFAULT,
+				nullptr, &pOut);
+			if (hr != S_OK) {
+				Status(L"Update failed: Could not determine AppData "
+				       L"location");
+				return false;
+			}
+
+			StringCbCopy(lpAppDataPath, sizeof(lpAppDataPath),
+				     pOut);
+		}
 	}
 
 	StringCbCat(lpAppDataPath, sizeof(lpAppDataPath), L"\\obs-studio");
@@ -1410,6 +1424,40 @@ static bool Update(wchar_t *cmdLine)
 		}
 	}
 
+	/* ------------------------------------- *
+	 * Install virtual camera                */
+
+	if (!bIsPortable) {
+		wchar_t regsvr[MAX_PATH];
+		wchar_t src[MAX_PATH];
+		wchar_t tmp[MAX_PATH];
+		wchar_t tmp2[MAX_PATH];
+
+		SHGetFolderPathW(nullptr, CSIDL_SYSTEM, nullptr,
+				 SHGFP_TYPE_CURRENT, regsvr);
+		StringCbCat(regsvr, sizeof(regsvr), L"\\regsvr32.exe");
+
+		GetCurrentDirectoryW(_countof(src), src);
+		StringCbCat(src, sizeof(src),
+			    L"\\data\\obs-plugins\\win-dshow\\");
+
+		StringCbCopy(tmp, sizeof(tmp), L"\"\"");
+		StringCbCat(tmp, sizeof(tmp), regsvr);
+		StringCbCat(tmp, sizeof(tmp), L"\" /s \"");
+		StringCbCat(tmp, sizeof(tmp), src);
+		StringCbCat(tmp, sizeof(tmp), L"obs-virtualcam-module");
+
+		StringCbCopy(tmp2, sizeof(tmp2), tmp);
+		StringCbCat(tmp2, sizeof(tmp2), L"32.dll\"\"");
+		_wsystem(tmp2);
+
+		if (is_64bit_windows()) {
+			StringCbCopy(tmp2, sizeof(tmp2), tmp);
+			StringCbCat(tmp2, sizeof(tmp2), L"64.dll\"\"");
+			_wsystem(tmp2);
+		}
+	}
+
 	/* ------------------------------------- *
 	 * Update hook files and vulkan registry */
 
@@ -1563,16 +1611,13 @@ static INT_PTR CALLBACK UpdateDialogProc(HWND hwnd, UINT message, WPARAM wParam,
 	return false;
 }
 
-static void RestartAsAdmin(LPWSTR lpCmdLine)
+static int RestartAsAdmin(LPCWSTR lpCmdLine, LPCWSTR cwd)
 {
 	wchar_t myPath[MAX_PATH];
 	if (!GetModuleFileNameW(nullptr, myPath, _countof(myPath) - 1)) {
-		return;
+		return 0;
 	}
 
-	wchar_t cwd[MAX_PATH];
-	GetCurrentDirectoryW(_countof(cwd) - 1, cwd);
-
 	SHELLEXECUTEINFO shExInfo = {0};
 	shExInfo.cbSize = sizeof(shExInfo);
 	shExInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
@@ -1588,16 +1633,27 @@ static void RestartAsAdmin(LPWSTR lpCmdLine)
 	 * windows :( */
 	AllowSetForegroundWindow(ASFW_ANY);
 
+	/* if the admin is a different user, save the path to the user's
+	 * appdata so we can load the correct manifest */
+	CoTaskMemPtr<wchar_t> pOut;
+	HRESULT hr = SHGetKnownFolderPath(FOLDERID_RoamingAppData,
+					  KF_FLAG_DEFAULT, nullptr, &pOut);
+	if (hr == S_OK)
+		SetEnvironmentVariable(L"OBS_USER_APPDATA_PATH", pOut);
+
 	if (ShellExecuteEx(&shExInfo)) {
 		DWORD exitCode;
 
+		WaitForSingleObject(shExInfo.hProcess, INFINITE);
+
 		if (GetExitCodeProcess(shExInfo.hProcess, &exitCode)) {
 			if (exitCode == 1) {
-				LaunchOBS();
+				return exitCode;
 			}
 		}
 		CloseHandle(shExInfo.hProcess);
 	}
+	return 0;
 }
 
 static bool HasElevation()
@@ -1622,11 +1678,36 @@ int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE, LPWSTR lpCmdLine, int)
 {
 	INITCOMMONCONTROLSEX icce;
 
+	wchar_t cwd[MAX_PATH];
+	wchar_t newPath[MAX_PATH];
+	GetCurrentDirectoryW(_countof(cwd) - 1, cwd);
+
+	is32bit = wcsstr(cwd, L"bin\\32bit") != nullptr;
+
 	if (!HasElevation()) {
+
+		WinHandle hMutex = OpenMutex(
+			SYNCHRONIZE, false, L"OBSUpdaterRunningAsNonAdminUser");
+		if (hMutex) {
+			MessageBox(
+				nullptr, L"Updater Error",
+				L"OBS Studio Updater must be run as an administrator.",
+				MB_ICONWARNING);
+			return 2;
+		}
+
 		HANDLE hLowMutex = CreateMutexW(
 			nullptr, true, L"OBSUpdaterRunningAsNonAdminUser");
 
-		RestartAsAdmin(lpCmdLine);
+		/* return code 1 =  user wanted to launch OBS */
+		if (RestartAsAdmin(lpCmdLine, cwd) == 1) {
+			StringCbCat(cwd, sizeof(cwd), L"\\..\\..");
+			GetFullPathName(cwd, _countof(newPath), newPath,
+					nullptr);
+			SetCurrentDirectory(newPath);
+
+			LaunchOBS();
+		}
 
 		if (hLowMutex) {
 			ReleaseMutex(hLowMutex);
@@ -1635,18 +1716,9 @@ int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE, LPWSTR lpCmdLine, int)
 
 		return 0;
 	} else {
-		{
-			wchar_t cwd[MAX_PATH];
-			wchar_t newPath[MAX_PATH];
-			GetCurrentDirectoryW(_countof(cwd) - 1, cwd);
-
-			is32bit = wcsstr(cwd, L"bin\\32bit") != nullptr;
-			StringCbCat(cwd, sizeof(cwd), L"\\..\\..");
-
-			GetFullPathName(cwd, _countof(newPath), newPath,
-					nullptr);
-			SetCurrentDirectory(newPath);
-		}
+		StringCbCat(cwd, sizeof(cwd), L"\\..\\..");
+		GetFullPathName(cwd, _countof(newPath), newPath, nullptr);
+		SetCurrentDirectory(newPath);
 
 		hinstMain = hInstance;
 
